<?php
/*
 * Copyright (c) 2005, Bull S.A..  All rights reserved.
 * Created by: Sebastien Decugis

 * This program is free software; you can redistribute it and/or modify it
 * under the terms of version 2 of the GNU General Public License as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it would be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write the Free Software Foundation, Inc., 59
 * Temple Place - Suite 330, Boston MA 02111-1307, USA.
 */


/* This file provides several functions used in other files.

-> interface to database
-> pre-built queries

*/

/* The next function will seek for a particular routine into the database.
 *
 * Input:
 *   $routine_name: Name of the routine being searched. If $exact_match is false, this name can be partial.
 *   $exact_match: if true, only routines with exact name are returned.
 *
 * Output:
 *   An array of found routines is returned:
 *   Array(
 *     <routine_name> = Array(
 *		"routine_id" => <routine_id> 
 * 		"routine_comment" => <routine_comment>
 *		)
 *	)
 */
function query_routines($routine_name="", $exact_match=0)
{
	$sql = "SELECT rou_id, rou_name, rou_comment FROM opts_routines";
	if ($routine_name != "")
	{
		$sql .= " WHERE rou_name LIKE ";
		if ($exact_match)
			$sql .= "'$routine_name'";
		else
			$sql .= "'%$routine_name%'";
	}
	$res = db_execute_select($sql);
	$result=array();
	
	foreach ($res as $data)
	{
		$result[$data["rou_name"]]=array(
			"routine_id"=>$data["rou_id"], 
			"routine_comment"=>$data["rou_comment"]);
	}
	return $result;
}	



/* The next function will retrieve assertions from the database (not bound to a particular OPTS release).
 *
 * Input:
 *   $routine_name: Name of the routine being searched. If $exact_match is false, this name can be partial.
 *   $exact_match: if true, only routines with exact name are returned.
 *
 * Output:
 *   An array of found assertions is returned:
 *   Array(
 *     <routine_name> = Array(
 *		<assert_id> => <assert_text>
 *		)
 *	)
 */
function query_all_asserts($routine_name="", $exact_match=0)
{
	$sql = "SELECT rou_name, assert_id, assert_text FROM opts_routines, opts_assertions WHERE rou_id=assert_routine";
	if ($routine_name != "")
	{
		$sql .= " AND rou_name LIKE ";
		if ($exact_match)
			$sql .= "'$routine_name'";
		else
			$sql .= "'%$routine_name%'";
	}
	$res = db_execute_select($sql);
	$result=array();
	
	foreach ($res as $data)
	{
		if (!isset($result[$data["rou_name"]]))
			$result[$data["rou_name"]]=array();
		$result[$data["rou_name"]][$data["assert_id"]]=$data["assert_text"];
	}
	return $result;
}	

/* The next function will retrieve OPTS release names from the database
 *
 * Input:
 *   $release_name: Name of the release being searched. If $exact_match is false, this name can be partial.
 *   $exact_match: if true, only releases with exact name are returned.
 *
 * Output:
 *   An array of found releases is returned:
 *   Array(
 *     <version_name> = Array(
 *		"ver_id" => <version_id>,
 *		"ver_comment" => <version_comment>
 *		)
 *	)
 */
function query_version($release_name="", $exact_match=0)
{
	$sql = "SELECT ver_id, ver_name, ver_comment, ver_module FROM opts_versions";
	if ($release_name != "")
	{
		$sql .= " WHERE ver_name LIKE ";
		if ($exact_match)
			$sql .= "'".addslashes($release_name)."'";
		else
			$sql .= "'%".addslashes($release_name)."%'";
	}
	$res = db_execute_select($sql);
	$result=array();
	
	foreach ($res as $data)
	{
		$result[$data["ver_name"]]= array(
			"ver_id"=>$data["ver_id"],
			"ver_comment"=>$data["ver_comment"],
			"ver_module" =>$data["ver_module"]);
	}
	return $result;
}

/* 
The next 2 functions are used to protect text data inserted and extracted from database.
It prevents user from putting malicious HTML code, for example.

A basic version will forbid all tags. Later a refined support can be developped.
*/

function stringToDB($string)
{
	return "'".addslashes($string)."'";
}

function stringFromDB($string)
{
	return nl2br(htmlentities($string));
}

?>
