<?php
/*
 * Copyright (c) 2005, Bull S.A..  All rights reserved.
 * Created by: Sebastien Decugis

 * This program is free software; you can redistribute it and/or modify it
 * under the terms of version 2 of the GNU General Public License as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it would be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write the Free Software Foundation, Inc., 59
 * Temple Place - Suite 330, Boston MA 02111-1307, USA.
 */

/* The goal of this file is to give detailed content of OPTS runs.

Inputs: 
-> run ID(s). 
  If one run ID is given, the full detail of the run is given.
  If 2 or more run IDs are given, a comparison is made.


Output:
-> html report of the differences between all the runs.


Future extensions:
 -> ability to select a subset of the routines only (with domains in the database)
 -> Summary of the results (X tests passed, Y test FAILED, ...)
 -> Ability to show only the summary
 -> Extend the "hide passed" function to something like "show only PASSED, show only HUNG" ...
    with the possibility to select what status to display (select from the summary, for example)

*/

//echo "<pre>\n";
//print_r($_POST);
//echo "</pre>\n";

$root="";
require($root."database.inc.php");
require($root."functions.inc.php");

$_PAGE["title"]="Reporting Tool";

require($root."header.inc.php");

/* Verify that the page was not opened directly */
if (!isset($_POST["opts_rel"]))
	die("<p>No valid parameter found. Please use <a href='run-browse.php'>this page</a>.</p>\n</body></html>\n");

/* We check for correct parameters */
$mode="";

if (isset($_POST["compare"]) && isset($_POST["chkd_runs"]))
{
	if (count($_POST["chkd_runs"]) <= 1)
		die("<p>Multiple releases comparison not implemented yet. <a href='run-browse.php'>Back</a></p>\n</body></html>\n");
	$mode="COMPARE";
}

$run_id = 0;

if (isset($_POST["browse"]))
{
	if (count($_POST["browse"]) != 1)
		die("<p>Invalid parameter supplied. <a href='run-browse.php'>Back</a></p>\n</body></html>\n");
	$mode="BROWSE";
	foreach ($_POST["browse"] as $key => $trash)
		$run_id = $key;
}


if ($mode =="")
	die("<p>No valid parameter found. Please use <a href='run-browse.php'>this page</a>.</p>\n</body></html>\n");


/* Remove unusefull POST variables */
if (isset($_POST["noasserts"]))
{
	unset($_POST["asserts"]);
	unset($_POST["noasserts"]);
}
if (isset($_POST["nodetails"]))
{
	unset($_POST["details"]);
	unset($_POST["nodetails"]);
}
if (isset($_POST["filter"]) && ($_POST["filter"]=="Show"))
{
	unset($_POST[$_POST["filtered"]]);
}
if (isset($_POST["filter"]))
	unset($_POST["filter"]);
if (isset($_POST["filtered"]))
	unset($_POST["filtered"]);
	
if (isset($_POST["nologs"]) || ($mode!="BROWSE"))
{
	unset($_POST["logs"]);
	unset($_POST["nologs"]);
}

/* Save the useful variables to allow page re-generation */
$POST_context="";
foreach ($_POST as $key => $val)
	if (is_array($_POST[$key]))
		foreach ($val as $subkey => $subval)
			$POST_context .="      <input type='hidden' name='".$key."[".$subkey."]' value='$subval'>\n";
	else
		$POST_context.="      <input type='hidden' name='$key' value='$val'>\n";

/* We have a correct set of parameters, so we show the control panel */
echo "   <form method=POST>\n";
echo $POST_context; // simplified session management

echo "    <table border=0 width=\"100%\">\n";
echo "      <tr>\n";
echo "        <td>Commands:</td>\n";
echo "	<td><input type='submit' name='reload' value='Reload'></td>\n";

/* Displayed only for mode COMPARE */
if ($mode == "COMPARE")
{
	if (isset($_POST["details"]))
		echo "	<td><input type='submit' name='nodetails' value='Hide common results'></td>\n";
	else
		echo "	<td><input type='submit' name='details' value='Show common results'></td>\n";
}

/* Displayed only for mode BROWSE */
if ($mode == "BROWSE")
{
	if (isset($_POST["logs"]))
		echo "	<td><input type='submit' name='nologs' value='Hide logs'></td>\n";
	else
		echo "	<td><input type='submit' name='logs' value='Show logs'></td>\n";
}

/* Displayed in both modes */
if (isset($_POST["asserts"]))
	echo "	<td><input type='submit' name='noasserts' value='Hide assertions'></td>\n";
else
	echo "	<td><input type='submit' name='asserts' value='Show assertions'></td>\n";

echo "        </tr>\n";
echo "      </table>\n";
echo "    </form>\n";

   


/* Whatever the mode is, we need to query the database with the same informations */
db_init();

$cond = "";
if ($mode == "BROWSE")
	$cond .= "res_run=$run_id";

if ($mode == "COMPARE")
{
	$cond .= " ( (0) ";
	foreach ($_POST["chkd_runs"] as $run_id)
		$cond .= " OR (res_run=$run_id) ";
	$cond .= " ) ";
}

$sql = " SELECT run_name, res_status, count(res_status) as cnt_status"
       ." FROM  opts_run, opts_run_results "
       ." WHERE opts_run.run_id = opts_run_results.res_run"
       ." AND ". $cond
       ." GROUP BY run_name, res_status";

db_query($sql, $stats_rs);
$stats=array();
while ($row = db_getline($stats_rs))
	$stats[$row["res_status"]][$row["run_name"]] = $row["cnt_status"];


$sql = " SELECT ver_name, run_name, res_run, assert_id, descr_id, rou_name, descr_info, res_status, "
       .(isset($_POST["logs"])?"res_log ":"res_log<>\"\" as is_res_log ")
       .(isset($_POST["asserts"])?", assert_text":"")
       ." FROM  opts_versions, opts_run, opts_routines, opts_run_results, opts_version_descriptions, "
       ."       opts_assertions "
       ." WHERE "
       ."       opts_run.run_id = opts_run_results.res_run"
       ." AND   opts_run_results.res_testcase = opts_version_descriptions.descr_id"
       ." AND   opts_version_descriptions.descr_version = opts_versions.ver_id"
       ." AND   opts_version_descriptions.descr_assert = opts_assertions.assert_id"
       ." AND   opts_assertions.assert_routine = opts_routines.rou_id "
       ." AND ". $cond;


//$res = db_execute_select($sql);
db_query($sql, $res);

/* Hash the results */
$data=array();
$asserts=array();
$runs=array();

//foreach ($res as $key => $row)
$displayed=0;
while ($row = db_getline($res))
{
	if ($mode != "BROWSE" || !isset($_POST[str_replace (' ','_',$row["res_status"])]))
	{
		$data[$row["rou_name"]][$row["assert_id"]][$row["descr_info"]][$row["descr_id"]][$row["run_name"]]=array(
			"status" => $row["res_status"],
			"log" => isset($_POST["logs"])?trim($row["res_log"]):$row["is_res_log"]);
		$displayed++;
	}
	$runs[$row["run_name"]]=array("id"=>$row["res_run"],"testsuite"=>$row["ver_name"]);
	if (isset($_POST["asserts"]))
		$asserts[$row["rou_name"]][$row["assert_id"]]=$row["assert_text"];
}

db_fini();


/* In case we are comparing several runs, we filter out the results which are the same for all runs */
if (($mode == "COMPARE") && !(isset($_POST["details"])))
{
	foreach ($data as $routine => $assertsdata)
		foreach ($assertsdata as $assert_id => $testdata)
			foreach ($testdata as $testname => $desc_data)
				foreach ($desc_data as $desc_id => $run_data)
				{
					$tmpres="";
					$diff=0;
					foreach ($run_data as $run_name => $detail)
					{
						if (!$tmpres) 
							$tmpres=$detail["status"]; // save the status of the 1st test
						elseif ($tmpres != $detail["status"])
							$diff=1; // we found a difference, we must not delete this one
					}
					if (!$diff)
					{
						unset ($data[$routine][$assert_id][$testname][$desc_id]); // No difference in this testcase
						$displayed--;
					}
					// we cascade-delete the empty tree branchs
					if (!$data[$routine][$assert_id][$testname]) unset($data[$routine][$assert_id][$testname]);
					if (!$data[$routine][$assert_id]) unset($data[$routine][$assert_id]);
					if (!$data[$routine]) unset($data[$routine]);
					if (!$data)
						die("The runs are identical -- aborted");
				}
}

ksort($runs);
ksort($data);

//echo "<pre>\n";
//print_r($data);
//echo "</pre>\n";



/* Print the statistics and filter command panel */

echo "   <table border='1'>\n";
echo "     <tr>\n";
echo "       <td align='center'><b><i>Statistics / status</i></b></td>\n";
foreach ($runs as $run => $dummy)
	echo "        <td align='center'><b>".stringFromDB($run)."</b></td>\n";
if ($mode == "BROWSE") 
	echo "       <td align='center'><b>Filtering</b></td>\n";
echo "     </tr>\n";
foreach ($stats as $status => $stats_data)
{
	echo "      <tr>\n";
	echo "        <td align='center'>";
	$font=0;
	if (ereg("(PASS|SKIP)", $status))
	{
		echo "<font color='green'>";
		$font=1;
	}
	if (ereg("(FAIL|INTERRUPTED|HUNG)", $status))
	{
		echo "<font color='red'>";
		$font=1;
	}
	if (ereg("(UNRESOLVED)", $status))
	{
		echo "<font color='orange'>";
		$font=1;
	}
	if (ereg("(UNSUPPORTED|UNTESTED)", $status))
	{
		echo "<font color='black'>";
		$font=1;
	}
	echo stringFromDB($status);
	if ($font)
		echo "</font>";
	
	echo "</td>\n";
	foreach ($runs as $run => $dummy)
	{
		echo "        <td align='center'>";
		if (isset($stats_data[$run]))
			echo $stats_data[$run];
		else
			echo "0";
		echo "</td>\n";
	}
	if ($mode == "BROWSE")
	{
		echo "       <td align='center'>\n";
		echo "         <form method='POST'>\n";
		echo $POST_context;
		echo "         <input type='hidden' name='".str_replace (' ','_',$status)."' value='hidden'>\n";
		echo "         <input type='hidden' name='filtered' value='".str_replace (' ','_',$status)."'>\n";
		echo "         <input type='submit' name='filter' value='";
		if (isset($_POST[str_replace (' ','_',$status)]))
			echo "Show";
		else
			echo "Hide";
		echo "'>\n";
		echo "         </form>\n";
		echo "       </td>\n";
	}
	echo "      </tr>\n";
}
echo "      <tr>\n";
echo "        <td colspan='".(count($runs)+(($mode=="BROWSE")?2:1))."'>";
echo "          <i>Currently displayed: <b>$displayed</b></td>\n";
echo "      </tr>\n";
echo "    </table>\n";
echo "  </form>\n";
echo " <hr width='30%'>\n";



/* Print the run details */


echo "    <table border='1'>\n";
echo "      <tr>\n";
echo "        <td><b>Routine</b></td>\n";
echo "        <td><b>File</b></td>\n";
foreach ($runs as $run => $trash)
	echo "        <td><b>".stringFromDB($run)."</b></td>\n";
	
if (isset($_POST["logs"]))
	echo "        <td><b>Log</b></td>\n";
if (isset($_POST["asserts"]))
	echo "        <td><b>Assertion detail</b></td>\n";
echo "      </tr>\n";

/* Display results */
$anomalies=0;
foreach ($data as $routine => $rou_data)
{
	echo "      <tr>\n";
	echo "        <td";
	$count=0;
	foreach($rou_data as $assert_data)
		$count += count($assert_data);
	if ($count > 1)
		echo " rowspan='".$count."'";
	echo ">".stringFromDB($routine)."</td>\n";
	$first=1;
	$anomalie=0;
	foreach ($rou_data as $assert_id => $assert_data)
	{
		$assert_first=1;
		$countTC = count($assert_data);
		foreach ($assert_data as $testcase => $test_data)
		{
			if (count($test_data) > 1)
			{
				$anomalie=1;
				$anomalies++;
			}
			foreach ($test_data as $testid => $run_data)
			{
				if (!$first)
					echo "      <tr>\n";
				echo "        <td>".stringFromDB($testcase).($anomalie?" (*)":"")."</td>\n";
				foreach ($runs as $run => $run_data_bis)
				{
					echo "        <td>";
					if (!isset($run_data[$run]))
						echo "&nbsp;";
					else
					{
						$font=0;
						echo "<a href=\"detailed.php?run_id=".$run_data_bis["id"]."&testcase_id=".$testid."\">";
						if (ereg("(PASS|SKIP)", $run_data[$run]["status"]))
						{
							echo "<font color='green'>";
							$font=1;
						}
						if (ereg("(FAIL|INTERRUPTED|HUNG)", $run_data[$run]["status"]))
						{
							echo "<font color='red'>";
							$font=1;
						}
						if (ereg("(UNRESOLVED)", $run_data[$run]["status"]))
						{
							echo "<font color='orange'>";
							$font=1;
						}
						if (ereg("(UNSUPPORTED|UNTESTED)", $run_data[$run]["status"]))
						{
							echo "<font color='black'>";
							$font=1;
						}
						echo stringFromDB($run_data[$run]["status"]);
						if ((!isset($_POST["logs"])) && $run_data[$run]["log"])
							echo " (log)";
						if ($font) echo "</font>";
						echo "</a>";
					}
					echo "</td>\n";
					if (isset($_POST["logs"]))
					{
						echo "         <td><pre>";
						echo strip_tags(stringFromDB($run_data[$run]["log"]));
						echo "&nbsp;</pre></td>\n";
					}
				}
				if (isset($_POST["asserts"]) && $assert_first)
					echo "        <td".($countTC>1?" rowspan=$countTC":"")."><pre>".strip_tags(stringFromDB(wordwrap($asserts[$routine][$assert_id])))."</pre></td>\n";
				echo "      </tr>\n";
				$first=0;
				$assert_first=0;
			}
		}
	}
}
echo "    </table>\n";

unset($data);
unset($runs);


require($root."footer.inc.php");
?>

