<?php
/*
 * Copyright (c) 2005, Bull S.A..  All rights reserved.
 * Created by: Sebastien Decugis

 * This program is free software; you can redistribute it and/or modify it
 * under the terms of version 2 of the GNU General Public License as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it would be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write the Free Software Foundation, Inc., 59
 * Temple Place - Suite 330, Boston MA 02111-1307, USA.
 */

/* This file allows the user to find which runs are in the database, 
  and to select for details or comparison.

It also provides an interface to enter new results or new testsuite release.

*/

/* We need to find all runs inside the database */
$root="";
require($root."database.inc.php");
require($root."functions.inc.php");
db_init();

$_PAGE["title"]="Results browser";

require($root."header.inc.php");

echo "<p>Go to <a href=\"".$root."admin/\">Administration Interface</a></p>\n";

if (isset($_GET["deselect_module"]))
	unset($_SESSION["module"]);

/* Count runs by modules in the database */
$sql = " SELECT DISTINCT ver_module, ver_id, res_run"
      ." FROM opts_versions, opts_version_descriptions, opts_run_results"
      ." WHERE ver_id=descr_version"
      ."   AND res_testcase=descr_id";
$runs = db_execute_select($sql);

$infos=array();
foreach ($runs as $row)
	if (isset($infos[$row["ver_module"]]))
		$infos[$row["ver_module"]]++;
	else
		$infos[$row["ver_module"]]=1;

if (isset($_GET["module"]))
	if (isset($infos[$_GET["module"]]))
		$_SESSION["module"]=$_GET["module"];

/* If we found no showable results */
if (count($infos)==0)
{
	echo "<p>There is no displayable result in the database yet.\n";
	echo "You may add some from the Administration Interface.</p>";
	db_fini();
	require($root."footer.inc.php");
	die();
}

/* If we have only 1 module, we don't need to ask the user */
if (count($infos)==1)
	$_SESSION["module"]=$runs[0]["ver_module"];
	
/* Otherwise, the user need to choose a module */
if (count($infos)>1)
{
	if (!isset($_SESSION["module"]))
	{
		/* Show modules list and let the user choose one */
		echo "<p>Please choose a module from the list:</p>\n";
		echo "<ul>\n";
		foreach ($infos as $module => $cnt)
		{
			echo "<li><a href=\"".$_SERVER["PHP_SELF"]."?module=$module\">";
			echo "<b>$module</b></a> ($cnt logfiles)</li>\n";
		}
		echo "</ul>\n";
		db_fini();
		require($root."footer.inc.php");
		die();
	}
	else
	{
		/* Show selected module and let the user cancel its selection */
		echo "<p>You're viewing the <i>".$_SESSION["module"]."</i> results currently.\n";
		echo "<a href=\"".$_SERVER["PHP_SELF"]."?deselect_module=1\">";
		echo "Click here</a> to change.</p>\n";
	}
}


/* Now, proceed to the display */

/*

We could achieve the same goal with an "outer join" sql request,
but it takes more time than 2 separate requests.

$sql= " SELECT DISTINCT ver_name,run_name "
     ." FROM "
     ."     ((opts_run_results INNER JOIN opts_run ON run_id=res_run)"
     ." RIGHT OUTER JOIN opts_version_descriptions "
     ."       ON descr_id=res_testcase), opts_versions"
     ." WHERE "
     ."   ver_id=descr_version";
*/
$sql= " SELECT DISTINCT ver_name, ver_comment, ver_id "
     ." FROM opts_versions, opts_version_descriptions"
     ." WHERE ver_id = descr_version "
     ."   AND ver_module = ".stringToDB($_SESSION["module"])
     ." ORDER BY ver_id DESC";
$release_table = db_execute_select($sql);


$sql= " SELECT DISTINCT run_name, run_comments, ver_name, run_id "
     ." FROM opts_run, opts_run_results, opts_version_descriptions, opts_versions "
     ." WHERE run_id=res_run"
     ."   AND res_testcase=descr_id"
     ."   AND descr_version=ver_id"
     ."   AND ver_module = ".stringToDB($_SESSION["module"])
     ." ORDER BY ver_name";
$run_table = db_execute_select($sql);

/* Merge the data from both table */
$infos=array();

foreach ($release_table as $release)
	$infos[$release["ver_name"]]=array(
					"id"=>$release["ver_id"],
					"comment"=>$release["ver_comment"],
					"runs"=>array());

foreach ($run_table as $run)
	$infos[$run["ver_name"]]["runs"][$run["run_id"]]=array(
					"name"=>$run["run_name"],
					"comment"=>$run["run_comments"]);

unset($release_table);
unset($run_table);

/* Ok, now we'll display the result on the html page */
?>

    <h3>Available results for the <?php echo $_SESSION["module"]; ?> module</h3>

    <table border="1">
      <tr>
        <td valign='top' align='center'><b>TestSuite Release</b></td>
	<td valign='top' align='center' colspan="2"><b>Run Designation</b></td>
	<td valign='top' align='center'><b>Action</b></td>
	<td valign='top'><b>Comments</b></td>
      </tr>
<?php
    if (!$infos)
    {
	    /* This should never happen */
?>
      <tr>
        <td colspan=5><i>Sorry, no data was found.</i></td>
      </tr>
<?php
    }
    else
    {
	foreach($infos as $release => $data)
	{
		echo "	        <tr>\n";
		echo "		  <td valign=\"top\"";
		if (count($data["runs"])>1)
			echo " rowspan=\"".(count($data["runs"])+2)."\"";
		elseif (count($data["runs"])==1)
			echo " rowspan=\"2\"";
		echo ">\n";
		echo "              <font color='orange'>".stringFromDB($release)."</font>\n";
		echo "            </td>\n";
		echo "            <td colspan='4' align=left>\n";
		echo "              <i><font color='orange'>".stringFromDB($data["comment"])."</font></i>\n";
		echo "            </td>\n";
		echo "          </tr>\n";

		if ($data["runs"])
		{
			echo "         <form method='POST' action='report.php'>\n";
			echo "          <input type='hidden' name='opts_rel' value=\"".$data["id"]."\">\n";

			/* default to masking PASS and SKIP status */
			echo "          <input type='hidden' name='build_PASS' value='hidden'>\n";
			echo "          <input type='hidden' name='link_SKIP' value='hidden'>\n";
			echo "          <input type='hidden' name='execution_PASS' value='hidden'>\n";
			
			foreach ($data["runs"] as $run_id => $run_data)
			{
				echo "		<tr>\n";
				echo "		  <td align=center>\n";
				if (count($data["runs"])>1)
					echo "              <input type='checkbox' name='chkd_runs[]' value='".$run_id."'>\n";
				else
					echo "              &nbsp;\n";
				echo "            </td>\n";
				echo "            <td>\n";
				echo "              ".stringFromDB($run_data["name"])."\n";
				echo "            </td>\n";
				echo "            <td align=center>\n";
				echo "              <input type='submit' name='browse[$run_id]' value='Browse'>\n";
				echo "            </td>\n";
				echo "            <td>\n";
				echo "              <i>".stringFromDB($run_data["comment"])."</i>\n";
				echo "            </td>\n";
				echo "          </tr>\n";
			}
			if (count($data["runs"])>1)
			{
				echo "          <tr>\n";
				echo "            <td colspan='3'  align=center>\n";
				echo "              <input type='submit' name='compare' value='Compare selected'>\n";
				echo "            </td>\n";
				echo "            <td>\n";
				echo "              &nbsp;\n";
				echo "            </td>\n";
				echo "          </tr>\n";
			}
			echo "         </form>\n";
		}
	}
    }
?>
    </table>
<?php
db_fini();
require($root."footer.inc.php");
?>
